@@ -3,10 +3,15 @@
 # Set encoding default for python 2.7
 # vim: syntax=python noexpandtab
 
+# ToDo
+# 1. unlink osdesc config symlink
+
+
 # gadgetconfig configures and enables a Gadget USB configuration
 #
 
 import os
+import sys
 import fnmatch
 
 """remove.py: ..."""
@@ -16,29 +21,34 @@ import fnmatch
 
 class RemoveGadget(object):
 
-	def __init__(self, configpath, manage):
+	def __init__(self, configpath, manage, verbose = False):
 		self.configpath = configpath
 		self.m = manage
+		self.verbose = verbose
+
+	def vprint(self, s):
+		if self.verbose:
+			print(s, file=sys.stderr)
 
 	def rmdir(self, path):
 		if not os.path.isdir(path):
-			print("rmdir: %s NOT A DIR" % (path))
+			self.vprint("rmdir: %s NOT A DIR" % (path))
 			return
-		print("rmdir: %s" % (path))
+		self.vprint("rmdir: %s" % (path))
 		os.rmdir(path)
 
 	def unlink(self, path):
 		if not os.path.islink(path):
-			print("rmdir: %s NOT A SYMLINK" % (path))
+			self.vprint("rmdir: %s NOT A SYMLINK" % (path))
 			return
-		print("unlink: %s" % (path))
+		self.vprint("unlink: %s" % (path))
 		os.unlink(path)
 
 	def listdir(self, path):
 		if not os.path.isdir(path):
-			print("listdir: %s NOT A DIR" % (path))
+			self.vprint("listdir: %s NOT A DIR" % (path))
 			return
-		print("listdir: %s" % (path))
+		self.vprint("listdir: %s" % (path))
 		return os.listdir(path)
 
 	def remove_strings(self, path):
@@ -53,10 +63,12 @@ class RemoveGadget(object):
 	# remove_device
 	#
 	# C.f. gadget_configfs.txt - section 7. Cleaning up
+	# 0. unlink os_desc symlink
 	# For each configuration .../configs/*
 	# 1. unlink Functions
 	# 2. rmdir strings/<lang>
 	# 3. rmdir config
+	# Then
 	# 4. rmdir functions/*
 	# 5. rmdir strings/<lang>
 	# 6. rmdir device
@@ -86,6 +98,14 @@ class RemoveGadget(object):
 			print("%s PERMISSION ERROR" % (device_path))
 			exit(1)
 
+		# step 0 - check for osdesc config symlink
+		#
+		os_desc_path = "%s/os_desc" % (device_path)
+		for entry in self.listdir(os_desc_path):
+			entry_path = "%s/%s" % (os_desc_path, entry)
+			if os.path.islink(entry_path):
+				self.unlink(entry_path)
+
 		# iterate across device path configs directory to handle
 		# steps #1, #2 and #3 for each configuration
 		#
@@ -108,7 +128,7 @@ class RemoveGadget(object):
 		functions_path = "%s/functions" % (device_path)
 		for f in self.listdir(functions_path):
 
-			# N.B. mass_storage has sub-functions lun.0...lun.N, 
+			# N.B. mass_storage has sub-functions lun.0...lun.N,
 			# lun.1...lun.N must be removed, lun.0 cannot be removed
 			sfunctions_path = "%s/%s" % (functions_path, f)
 			for d in self.listdir("%s/%s" % (functions_path, f)):
